@@ -1,6 +1,29 @@
-/* ************************************************************************
- * Copyright (c) 2016-2023 Advanced Micro Devices, Inc.
- * ************************************************************************ */
+/* **************************************************************************
+ * Copyright (C) 2016-2023 Advanced Micro Devices, Inc. All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ *
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+ * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+ * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
+ * *************************************************************************/
 
 #include <fmt/core.h>
 #include <fmt/ostream.h>
@@ -339,7 +362,7 @@ try
             "                           Stride for matrices/vectors Z.\n"
             "                           ")
 
-        // refactorization size options
+        // refactorization options
         ("nnzM",
          value<rocblas_int>(),
             "The number of non-zero elements in sparse matrix M.\n"
@@ -375,6 +398,12 @@ try
             "                           The benchmark client will use the available case closest to the input value.\n"
             "                           ")
 
+        ("rfinfo_mode",
+         value<char>(),
+            "Specifies the desired re-factorization algorithm.\n"
+            "                           1 = LU, 2 = Cholesky.\n"
+            "                           ")
+
         // bdsqr options
         ("nc",
          value<rocblas_int>()->default_value(0),
@@ -624,6 +653,7 @@ try
     argus.validate_eorder("eorder");
     argus.validate_esort("esort");
     argus.validate_itype("itype");
+    argus.validate_rfinfo_mode("rfinfo_mode");
 
     // prepare logging infrastructure and ignore environment variables
     rocsolver_log_begin();
