@@ -1,6 +1,29 @@
-/* ************************************************************************
- * Copyright (c) 2022-2023 Advanced Micro Devices, Inc.
- * ************************************************************************ */
+/* **************************************************************************
+ * Copyright (C) 2022-2023 Advanced Micro Devices, Inc. All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ *
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+ * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+ * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
+ * *************************************************************************/
 
 #pragma once
 
@@ -233,8 +256,28 @@ void bdsvdx_getError(const rocblas_handle handle,
         // U is stored in hZRes, and V is stored in hZRes+n
         if(svect != rocblas_svect_none)
         {
-            err = 0;
+            // U and V should be orthonormal, if they are then U^T*U and V^T*V should be the identity
+            if(nn > 0)
+            {
+                std::vector<T> UUres(nn * nn, 0.0);
+                std::vector<T> VVres(nn * nn, 0.0);
+                std::vector<T> I(nn * nn, 0.0);
 
+                for(rocblas_int i = 0; i < nn; i++)
+                    I[i + i * nn] = T(1);
+
+                cpu_gemm(rocblas_operation_conjugate_transpose, rocblas_operation_none, nn, nn, n,
+                         T(1), hZRes[0], ldz, hZRes[0], ldz, T(0), UUres.data(), nn);
+                err = norm_error('F', nn, nn, nn, I.data(), UUres.data());
+                *max_err = err > *max_err ? err : *max_err;
+
+                cpu_gemm(rocblas_operation_conjugate_transpose, rocblas_operation_none, nn, nn, n,
+                         T(1), hZRes[0] + n, ldz, hZRes[0] + n, ldz, T(0), VVres.data(), nn);
+                err = norm_error('F', nn, nn, nn, I.data(), VVres.data());
+                *max_err = err > *max_err ? err : *max_err;
+            }
+
+            err = 0;
             // form bidiagonal matrix B
             std::vector<T> B(n * n);
             for(rocblas_int i = 0; i < n; i++)
@@ -381,16 +424,19 @@ void testing_bdsvdx(Arguments& argus)
     char uploC = argus.get<char>("uplo");
     char svectC = argus.get<char>("svect");
     char srangeC = argus.get<char>("srange");
-    rocblas_int n = argus.get<rocblas_int>("n");
+    rocblas_fill uplo = char2rocblas_fill(uploC);
+    rocblas_svect svect = char2rocblas_svect(svectC);
+    rocblas_srange srange = char2rocblas_srange(srangeC);
+
     T vl = T(argus.get<double>("vl", 0));
     T vu = T(argus.get<double>("vu", srangeC == 'V' ? 1 : 0));
     rocblas_int il = argus.get<rocblas_int>("il", srangeC == 'I' ? 1 : 0);
     rocblas_int iu = argus.get<rocblas_int>("iu", srangeC == 'I' ? 1 : 0);
-    rocblas_int ldz = argus.get<rocblas_int>("ldz", 2 * n);
 
-    rocblas_fill uplo = char2rocblas_fill(uploC);
-    rocblas_svect svect = char2rocblas_svect(svectC);
-    rocblas_srange srange = char2rocblas_srange(srangeC);
+    rocblas_int n = argus.get<rocblas_int>("n");
+    rocblas_int ldz = argus.get<rocblas_int>("ldz", 2 * n);
+    rocblas_int nsv_max = (srange == rocblas_srange_index ? iu - il + 1 : n);
+
     rocblas_int hot_calls = argus.iters;
 
     // check non-supported values
@@ -412,8 +458,9 @@ void testing_bdsvdx(Arguments& argus)
     // determine sizes
     size_t size_D = n;
     size_t size_E = n;
-    size_t size_S = n;
-    size_t size_Z = ldz * n;
+    size_t size_S = nsv_max;
+    size_t size_S_cpu = n;
+    size_t size_Z = ldz * nsv_max;
     size_t size_Ifail = n;
     double max_error = 0, gpu_time_used = 0, cpu_time_used = 0;
 
@@ -465,7 +512,7 @@ void testing_bdsvdx(Arguments& argus)
     // host
     host_strided_batch_vector<T> hD(size_D, 1, size_D, 1);
     host_strided_batch_vector<T> hE(size_E, 1, size_E, 1);
-    host_strided_batch_vector<T> hS(size_S, 1, size_S, 1);
+    host_strided_batch_vector<T> hS(size_S_cpu, 1, size_S_cpu, 1); // extra space for cpu_bdsvdx
     host_strided_batch_vector<T> hSRes(size_SRes, 1, size_SRes, 1);
     host_strided_batch_vector<T> hZ(size_Z, 1, size_Z, 1);
     host_strided_batch_vector<T> hZRes(size_ZRes, 1, size_ZRes, 1);
@@ -523,9 +570,9 @@ void testing_bdsvdx(Arguments& argus)
                               argus.perf);
 
     // validate results for rocsolver-test
-    // using n * machine_precision as tolerance
+    // using 2 * n * machine_precision as tolerance
     if(argus.unit_check)
-        ROCSOLVER_TEST_CHECK(T, max_error, n);
+        ROCSOLVER_TEST_CHECK(T, max_error, 2 * n);
 
     // output results for rocsolver-bench
     if(argus.timing)
