@@ -1,6 +1,29 @@
-/* ************************************************************************
- * Copyright (c) 2023 Advanced Micro Devices, Inc.
- * ************************************************************************ */
+/* **************************************************************************
+ * Copyright (C) 2023 Advanced Micro Devices, Inc. All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ *
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+ * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+ * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
+ * *************************************************************************/
 
 #pragma once
 
@@ -15,6 +38,7 @@
 template <typename T>
 void csrrf_analysis_checkBadArgs(rocblas_handle handle,
                                  const rocblas_int n,
+                                 const rocblas_int nrhs,
                                  const rocblas_int nnzM,
                                  rocblas_int* ptrM,
                                  rocblas_int* indM,
@@ -25,15 +49,10 @@ void csrrf_analysis_checkBadArgs(rocblas_handle handle,
                                  T valT,
                                  rocblas_int* pivP,
                                  rocblas_int* pivQ,
+                                 T B,
+                                 const rocblas_int ldb,
                                  rocsolver_rfinfo rfinfo)
 {
-    // TODO: later we can extend the test for the solver phase as well
-    // (it's not really needed as the analysis is executed before calling the solver
-    // in csrrf_solve test anyways)
-    rocblas_int ldb = n;
-    rocblas_int nrhs = 0;
-    T B = nullptr;
-
     // handle
     EXPECT_ROCBLAS_STATUS(rocsolver_csrrf_analysis(nullptr, n, nrhs, nnzM, ptrM, indM, valM, nnzT,
                                                    ptrT, indT, valT, pivP, pivQ, B, ldb, rfinfo),
@@ -104,8 +123,10 @@ void testing_csrrf_analysis_bad_arg()
     rocblas_local_handle handle;
     rocsolver_local_rfinfo rfinfo(handle);
     rocblas_int n = 1;
+    rocblas_int nrhs = 1;
     rocblas_int nnzM = 1;
     rocblas_int nnzT = 1;
+    rocblas_int ldb = 1;
 
     // memory allocations
     device_strided_batch_vector<rocblas_int> ptrM(1, 1, 1, 1);
@@ -116,6 +137,7 @@ void testing_csrrf_analysis_bad_arg()
     device_strided_batch_vector<T> valT(1, 1, 1, 1);
     device_strided_batch_vector<rocblas_int> pivP(1, 1, 1, 1);
     device_strided_batch_vector<rocblas_int> pivQ(1, 1, 1, 1);
+    device_strided_batch_vector<T> B(1, 1, 1, 1);
     CHECK_HIP_ERROR(ptrM.memcheck());
     CHECK_HIP_ERROR(indM.memcheck());
     CHECK_HIP_ERROR(valM.memcheck());
@@ -124,16 +146,18 @@ void testing_csrrf_analysis_bad_arg()
     CHECK_HIP_ERROR(valT.memcheck());
     CHECK_HIP_ERROR(pivP.memcheck());
     CHECK_HIP_ERROR(pivQ.memcheck());
+    CHECK_HIP_ERROR(B.memcheck());
 
     // check bad arguments
-    csrrf_analysis_checkBadArgs(handle, n, nnzM, ptrM.data(), indM.data(), valM.data(), nnzT,
+    csrrf_analysis_checkBadArgs(handle, n, nrhs, nnzM, ptrM.data(), indM.data(), valM.data(), nnzT,
                                 ptrT.data(), indT.data(), valT.data(), pivP.data(), pivQ.data(),
-                                rfinfo);
+                                B.data(), ldb, rfinfo);
 }
 
 template <bool CPU, bool GPU, typename T, typename Td, typename Ud, typename Th, typename Uh>
 void csrrf_analysis_initData(rocblas_handle handle,
                              const rocblas_int n,
+                             const rocblas_int nrhs,
                              const rocblas_int nnzA,
                              Ud& dptrA,
                              Ud& dindA,
@@ -144,6 +168,8 @@ void csrrf_analysis_initData(rocblas_handle handle,
                              Td& dvalT,
                              Ud& dpivP,
                              Ud& dpivQ,
+                             Td& dB,
+                             const rocblas_int ldb,
                              Uh& hptrA,
                              Uh& hindA,
                              Th& hvalA,
@@ -152,7 +178,9 @@ void csrrf_analysis_initData(rocblas_handle handle,
                              Th& hvalT,
                              Uh& hpivP,
                              Uh& hpivQ,
-                             const fs::path testcase)
+                             Th& hB,
+                             const fs::path testcase,
+                             const rocsolver_rfinfo_mode mode)
 {
     if(CPU)
     {
@@ -175,12 +203,22 @@ void csrrf_analysis_initData(rocblas_handle handle,
         read_matrix(file.string(), 1, nnzT, hvalT.data(), 1);
 
         // read-in P
-        file = testcase / "P";
-        read_matrix(file.string(), 1, n, hpivP.data(), 1);
+        if(mode == rocsolver_rfinfo_mode_lu)
+        {
+            file = testcase / "P";
+            read_matrix(file.string(), 1, n, hpivP.data(), 1);
+        }
 
         // read-in Q
         file = testcase / "Q";
         read_matrix(file.string(), 1, n, hpivQ.data(), 1);
+
+        // read-in B
+        if(nrhs > 0)
+        {
+            file = testcase / fs::path(fmt::format("B_{}", nrhs));
+            read_matrix(file.string(), n, nrhs, hB.data(), ldb);
+        }
     }
 
     if(GPU)
@@ -191,14 +229,22 @@ void csrrf_analysis_initData(rocblas_handle handle,
         CHECK_HIP_ERROR(dptrT.transfer_from(hptrT));
         CHECK_HIP_ERROR(dindT.transfer_from(hindT));
         CHECK_HIP_ERROR(dvalT.transfer_from(hvalT));
-        CHECK_HIP_ERROR(dpivP.transfer_from(hpivP));
+
+        if(mode == rocsolver_rfinfo_mode_lu)
+            CHECK_HIP_ERROR(dpivP.transfer_from(hpivP));
+        else
+            CHECK_HIP_ERROR(dpivP.transfer_from(hpivQ));
         CHECK_HIP_ERROR(dpivQ.transfer_from(hpivQ));
+
+        if(nrhs > 0)
+            CHECK_HIP_ERROR(dB.transfer_from(hB));
     }
 }
 
 template <typename T, typename Td, typename Ud, typename Th, typename Uh>
 void csrrf_analysis_getError(rocblas_handle handle,
                              const rocblas_int n,
+                             const rocblas_int nrhs,
                              const rocblas_int nnzM,
                              Ud& dptrM,
                              Ud& dindM,
@@ -209,6 +255,8 @@ void csrrf_analysis_getError(rocblas_handle handle,
                              Td& dvalT,
                              Ud& dpivP,
                              Ud& dpivQ,
+                             Td& dB,
+                             const rocblas_int ldb,
                              rocsolver_rfinfo rfinfo,
                              Uh& hptrM,
                              Uh& hindM,
@@ -218,26 +266,21 @@ void csrrf_analysis_getError(rocblas_handle handle,
                              Th& hvalT,
                              Uh& hpivP,
                              Uh& hpivQ,
+                             Th& hB,
                              double* max_err,
-                             const fs::path testcase)
+                             const fs::path testcase,
+                             const rocsolver_rfinfo_mode mode)
 {
-    // TODO: later we can extend the test for the solver phase as well
-    // (it's not really needed as the analysis is executed before calling the solver
-    // in csrrf_solve test anyways)
-    rocblas_int ldb = n;
-    rocblas_int nrhs = 0;
-    T* B = nullptr;
-
     // input data initialization
-    csrrf_analysis_initData<true, true, T>(handle, n, nnzM, dptrM, dindM, dvalM, nnzT, dptrT, dindT,
-                                           dvalT, dpivP, dpivQ, hptrM, hindM, hvalM, hptrT, hindT,
-                                           hvalT, hpivP, hpivQ, testcase);
+    csrrf_analysis_initData<true, true, T>(handle, n, nrhs, nnzM, dptrM, dindM, dvalM, nnzT, dptrT,
+                                           dindT, dvalT, dpivP, dpivQ, dB, ldb, hptrM, hindM, hvalM,
+                                           hptrT, hindT, hvalT, hpivP, hpivQ, hB, testcase, mode);
 
     // execute computations
     // GPU lapack
     CHECK_ROCBLAS_ERROR(rocsolver_csrrf_analysis(
         handle, n, nrhs, nnzM, dptrM.data(), dindM.data(), dvalM.data(), nnzT, dptrT.data(),
-        dindT.data(), dvalT.data(), dpivP.data(), dpivQ.data(), B, ldb, rfinfo));
+        dindT.data(), dvalT.data(), dpivP.data(), dpivQ.data(), dB.data(), ldb, rfinfo));
 
     // No error to calculate...
     // (analysis phase is required by refact and solve, so its results are validated there)
@@ -247,6 +290,7 @@ void csrrf_analysis_getError(rocblas_handle handle,
 template <typename T, typename Td, typename Ud, typename Th, typename Uh>
 void csrrf_analysis_getPerfData(rocblas_handle handle,
                                 const rocblas_int n,
+                                const rocblas_int nrhs,
                                 const rocblas_int nnzM,
                                 Ud& dptrM,
                                 Ud& dindM,
@@ -257,6 +301,8 @@ void csrrf_analysis_getPerfData(rocblas_handle handle,
                                 Td& dvalT,
                                 Ud& dpivP,
                                 Ud& dpivQ,
+                                Td& dB,
+                                const rocblas_int ldb,
                                 rocsolver_rfinfo rfinfo,
                                 Uh& hptrM,
                                 Uh& hindM,
@@ -266,37 +312,32 @@ void csrrf_analysis_getPerfData(rocblas_handle handle,
                                 Th& hvalT,
                                 Uh& hpivP,
                                 Uh& hpivQ,
+                                Th& hB,
                                 double* gpu_time_used,
                                 double* cpu_time_used,
                                 const rocblas_int hot_calls,
                                 const int profile,
                                 const bool profile_kernels,
                                 const bool perf,
-                                const fs::path testcase)
+                                const fs::path testcase,
+                                const rocsolver_rfinfo_mode mode)
 {
-    // TODO: later we can extend the test for the solver phase as well
-    // (it's not really needed as the analysis is executed before calling the solver
-    // in csrrf_solve test anyways)
-    rocblas_int ldb = n;
-    rocblas_int nrhs = 0;
-    T* B = nullptr;
-
     *cpu_time_used = nan(""); // no timing on cpu-lapack execution
 
-    csrrf_analysis_initData<true, false, T>(handle, n, nnzM, dptrM, dindM, dvalM, nnzT, dptrT,
-                                            dindT, dvalT, dpivP, dpivQ, hptrM, hindM, hvalM, hptrT,
-                                            hindT, hvalT, hpivP, hpivQ, testcase);
+    csrrf_analysis_initData<true, false, T>(handle, n, nrhs, nnzM, dptrM, dindM, dvalM, nnzT, dptrT,
+                                            dindT, dvalT, dpivP, dpivQ, dB, ldb, hptrM, hindM, hvalM,
+                                            hptrT, hindT, hvalT, hpivP, hpivQ, hB, testcase, mode);
 
     // cold calls
     for(int iter = 0; iter < 2; iter++)
     {
-        csrrf_analysis_initData<false, true, T>(handle, n, nnzM, dptrM, dindM, dvalM, nnzT, dptrT,
-                                                dindT, dvalT, dpivP, dpivQ, hptrM, hindM, hvalM,
-                                                hptrT, hindT, hvalT, hpivP, hpivQ, testcase);
+        csrrf_analysis_initData<false, true, T>(
+            handle, n, nrhs, nnzM, dptrM, dindM, dvalM, nnzT, dptrT, dindT, dvalT, dpivP, dpivQ, dB,
+            ldb, hptrM, hindM, hvalM, hptrT, hindT, hvalT, hpivP, hpivQ, hB, testcase, mode);
 
         CHECK_ROCBLAS_ERROR(rocsolver_csrrf_analysis(
             handle, n, nrhs, nnzM, dptrM.data(), dindM.data(), dvalM.data(), nnzT, dptrT.data(),
-            dindT.data(), dvalT.data(), dpivP.data(), dpivQ.data(), B, ldb, rfinfo));
+            dindT.data(), dvalT.data(), dpivP.data(), dpivQ.data(), dB.data(), ldb, rfinfo));
     }
 
     // gpu-lapack performance
@@ -316,14 +357,14 @@ void csrrf_analysis_getPerfData(rocblas_handle handle,
 
     for(rocblas_int iter = 0; iter < hot_calls; iter++)
     {
-        csrrf_analysis_initData<false, true, T>(handle, n, nnzM, dptrM, dindM, dvalM, nnzT, dptrT,
-                                                dindT, dvalT, dpivP, dpivQ, hptrM, hindM, hvalM,
-                                                hptrT, hindT, hvalT, hpivP, hpivQ, testcase);
+        csrrf_analysis_initData<false, true, T>(
+            handle, n, nrhs, nnzM, dptrM, dindM, dvalM, nnzT, dptrT, dindT, dvalT, dpivP, dpivQ, dB,
+            ldb, hptrM, hindM, hvalM, hptrT, hindT, hvalT, hpivP, hpivQ, hB, testcase, mode);
 
         start = get_time_us_sync(stream);
         rocsolver_csrrf_analysis(handle, n, nrhs, nnzM, dptrM.data(), dindM.data(), dvalM.data(),
                                  nnzT, dptrT.data(), dindT.data(), dvalT.data(), dpivP.data(),
-                                 dpivQ.data(), B, ldb, rfinfo);
+                                 dpivQ.data(), dB.data(), ldb, rfinfo);
         *gpu_time_used += get_time_us_sync(stream) - start;
     }
     *gpu_time_used /= hot_calls;
@@ -336,16 +377,15 @@ void testing_csrrf_analysis(Arguments& argus)
     rocblas_local_handle handle;
     rocsolver_local_rfinfo rfinfo(handle);
     rocblas_int n = argus.get<rocblas_int>("n");
+    rocblas_int nrhs = argus.get<rocblas_int>("nrhs", 0);
     rocblas_int nnzM = argus.get<rocblas_int>("nnzM");
     rocblas_int nnzT = argus.get<rocblas_int>("nnzT");
+    rocblas_int ldb = argus.get<rocblas_int>("ldb", n);
+    char modeC = argus.get<char>("rfinfo_mode", '1');
     rocblas_int hot_calls = argus.iters;
 
-    // TODO: later we can extend the test for the solver phase as well
-    // (it's not really needed as the analysis is executed before calling the solver
-    // in csrrf_solve test anyways)
-    rocblas_int ldb = n;
-    rocblas_int nrhs = 0;
-    T* B = nullptr;
+    rocsolver_rfinfo_mode mode = char2rocsolver_rfinfo_mode(modeC);
+    CHECK_ROCBLAS_ERROR(rocsolver_set_rfinfo_mode(rfinfo, mode));
 
     // check non-supported values
     // N/A
@@ -354,12 +394,12 @@ void testing_csrrf_analysis(Arguments& argus)
     bool invalid_size = (n < 0 || nnzM < 0 || nnzT < 0 || nrhs < 0 || ldb < n);
     if(invalid_size)
     {
-        EXPECT_ROCBLAS_STATUS(rocsolver_csrrf_analysis(handle, n, nrhs, nnzM, (rocblas_int*)nullptr,
-                                                       (rocblas_int*)nullptr, (T*)nullptr, nnzT,
-                                                       (rocblas_int*)nullptr, (rocblas_int*)nullptr,
-                                                       (T*)nullptr, (rocblas_int*)nullptr,
-                                                       (rocblas_int*)nullptr, B, ldb, rfinfo),
-                              rocblas_status_invalid_size);
+        EXPECT_ROCBLAS_STATUS(
+            rocsolver_csrrf_analysis(handle, n, nrhs, nnzM, (rocblas_int*)nullptr,
+                                     (rocblas_int*)nullptr, (T*)nullptr, nnzT, (rocblas_int*)nullptr,
+                                     (rocblas_int*)nullptr, (T*)nullptr, (rocblas_int*)nullptr,
+                                     (rocblas_int*)nullptr, (T*)nullptr, ldb, rfinfo),
+            rocblas_status_invalid_size);
 
         if(argus.timing)
             rocsolver_bench_inform(inform_invalid_size);
@@ -403,9 +443,28 @@ void testing_csrrf_analysis(Arguments& argus)
     fs::path testcase;
     if(n > 0)
     {
-        testcase = get_sparse_data_dir() / fs::path(fmt::format("mat_{}_{}", n, nnzM));
-        fs::path file = testcase / "ptrT";
-        read_last(file.string(), &nnzT);
+        std::string matname;
+        if(mode == rocsolver_rfinfo_mode_lu)
+            matname = fmt::format("mat_{}_{}", n, nnzM);
+        else
+            matname = fmt::format("posmat_{}_{}", n, nnzM);
+
+        testcase = get_sparse_data_dir() / fs::path(matname);
+        fs::path fileA = testcase / "ptrA";
+        read_last(fileA.string(), &nnzM);
+        fs::path fileT = testcase / "ptrT";
+        read_last(fileT.string(), &nnzT);
+    }
+
+    // determine existing right-hand-side
+    if(nrhs > 0)
+    {
+        if(nrhs <= 5)
+            nrhs = 1;
+        else if(nrhs <= 20)
+            nrhs = 10;
+        else
+            nrhs = 30;
     }
 
     // memory size query if necessary
@@ -415,7 +474,7 @@ void testing_csrrf_analysis(Arguments& argus)
         CHECK_ALLOC_QUERY(rocsolver_csrrf_analysis(
             handle, n, nrhs, nnzM, (rocblas_int*)nullptr, (rocblas_int*)nullptr, (T*)nullptr, nnzT,
             (rocblas_int*)nullptr, (rocblas_int*)nullptr, (T*)nullptr, (rocblas_int*)nullptr,
-            (rocblas_int*)nullptr, B, ldb, rfinfo));
+            (rocblas_int*)nullptr, (T*)nullptr, ldb, rfinfo));
 
         size_t size;
         CHECK_ROCBLAS_ERROR(rocblas_stop_device_memory_size_query(handle, &size));
@@ -437,6 +496,7 @@ void testing_csrrf_analysis(Arguments& argus)
     size_t size_valT = size_t(nnzT);
     size_t size_pivP = size_t(n);
     size_t size_pivQ = size_t(n);
+    size_t size_BX = size_t(ldb) * nrhs;
 
     double max_error = 0, gpu_time_used = 0, cpu_time_used = 0;
 
@@ -449,6 +509,7 @@ void testing_csrrf_analysis(Arguments& argus)
     host_strided_batch_vector<T> hvalT(size_valT, 1, size_valT, 1);
     host_strided_batch_vector<rocblas_int> hpivP(size_pivP, 1, size_pivP, 1);
     host_strided_batch_vector<rocblas_int> hpivQ(size_pivQ, 1, size_pivQ, 1);
+    host_strided_batch_vector<T> hB(size_BX, 1, size_BX, 1);
 
     device_strided_batch_vector<rocblas_int> dptrM(size_ptrM, 1, size_ptrM, 1);
     device_strided_batch_vector<rocblas_int> dindM(size_indM, 1, size_indM, 1);
@@ -458,6 +519,7 @@ void testing_csrrf_analysis(Arguments& argus)
     device_strided_batch_vector<T> dvalT(size_valT, 1, size_valT, 1);
     device_strided_batch_vector<rocblas_int> dpivP(size_pivP, 1, size_pivP, 1);
     device_strided_batch_vector<rocblas_int> dpivQ(size_pivQ, 1, size_pivQ, 1);
+    device_strided_batch_vector<T> dB(size_BX, 1, size_BX, 1);
     CHECK_HIP_ERROR(dptrM.memcheck());
     CHECK_HIP_ERROR(dptrT.memcheck());
     if(size_indM)
@@ -472,15 +534,17 @@ void testing_csrrf_analysis(Arguments& argus)
         CHECK_HIP_ERROR(dpivP.memcheck());
     if(size_pivQ)
         CHECK_HIP_ERROR(dpivQ.memcheck());
+    if(size_BX)
+        CHECK_HIP_ERROR(dB.memcheck());
 
     // check quick return
     if(n == 0)
     {
-        EXPECT_ROCBLAS_STATUS(rocsolver_csrrf_analysis(handle, n, nrhs, nnzM, dptrM.data(),
-                                                       dindM.data(), dvalM.data(), nnzT,
-                                                       dptrT.data(), dindT.data(), dvalT.data(),
-                                                       dpivP.data(), dpivQ.data(), B, ldb, rfinfo),
-                              rocblas_status_success);
+        EXPECT_ROCBLAS_STATUS(
+            rocsolver_csrrf_analysis(handle, n, nrhs, nnzM, dptrM.data(), dindM.data(),
+                                     dvalM.data(), nnzT, dptrT.data(), dindT.data(), dvalT.data(),
+                                     dpivP.data(), dpivQ.data(), dB.data(), ldb, rfinfo),
+            rocblas_status_success);
         if(argus.timing)
             rocsolver_bench_inform(inform_quick_return);
 
@@ -489,16 +553,17 @@ void testing_csrrf_analysis(Arguments& argus)
 
     // check computations
     if(argus.unit_check || argus.norm_check)
-        csrrf_analysis_getError<T>(handle, n, nnzM, dptrM, dindM, dvalM, nnzT, dptrT, dindT, dvalT,
-                                   dpivP, dpivQ, rfinfo, hptrM, hindM, hvalM, hptrT, hindT, hvalT,
-                                   hpivP, hpivQ, &max_error, testcase);
+        csrrf_analysis_getError<T>(handle, n, nrhs, nnzM, dptrM, dindM, dvalM, nnzT, dptrT, dindT,
+                                   dvalT, dpivP, dpivQ, dB, ldb, rfinfo, hptrM, hindM, hvalM, hptrT,
+                                   hindT, hvalT, hpivP, hpivQ, hB, &max_error, testcase, mode);
 
     // collect performance data
     if(argus.timing)
-        csrrf_analysis_getPerfData<T>(handle, n, nnzM, dptrM, dindM, dvalM, nnzT, dptrT, dindT,
-                                      dvalT, dpivP, dpivQ, rfinfo, hptrM, hindM, hvalM, hptrT, hindT,
-                                      hvalT, hpivP, hpivQ, &gpu_time_used, &cpu_time_used, hot_calls,
-                                      argus.profile, argus.profile_kernels, argus.perf, testcase);
+        csrrf_analysis_getPerfData<T>(handle, n, nrhs, nnzM, dptrM, dindM, dvalM, nnzT, dptrT,
+                                      dindT, dvalT, dpivP, dpivQ, dB, ldb, rfinfo, hptrM, hindM,
+                                      hvalM, hptrT, hindT, hvalT, hpivP, hpivQ, hB, &gpu_time_used,
+                                      &cpu_time_used, hot_calls, argus.profile,
+                                      argus.profile_kernels, argus.perf, testcase, mode);
 
     // validate results for rocsolver-test
     // N/A
@@ -509,8 +574,8 @@ void testing_csrrf_analysis(Arguments& argus)
         if(!argus.perf)
         {
             rocsolver_bench_header("Arguments:");
-            rocsolver_bench_output("n", "nnzM", "nnzT");
-            rocsolver_bench_output(n, nnzM, nnzT);
+            rocsolver_bench_output("n", "nrhs", "nnzM", "nnzT", "ldb");
+            rocsolver_bench_output(n, nrhs, nnzM, nnzT, ldb);
 
             rocsolver_bench_header("Results:");
             if(argus.norm_check)
