@@ -1,6 +1,29 @@
-/* ************************************************************************
- * Copyright (c) 2022-2023 Advanced Micro Devices, Inc.
- * ************************************************************************ */
+/* **************************************************************************
+ * Copyright (C) 2022-2023 Advanced Micro Devices, Inc. All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ *
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+ * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+ * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
+ * *************************************************************************/
 
 #pragma once
 
@@ -445,18 +468,14 @@ void gesvdx_getError(const rocblas_handle handle,
         }
     }*/
 
-    // Check number of returned singular values
     double err = 0;
     for(rocblas_int b = 0; b < bc; ++b)
     {
+        // check number of computed singular values
+        rocblas_int nn = hNsvRes[b][0];
+        *max_err += std::abs(nn - hNsv[b][0]);
         EXPECT_EQ(hNsv[b][0], hNsvRes[b][0]) << "where b = " << b;
-        if(hNsv[b][0] != hNsvRes[b][0])
-            err++;
-    }
-    *max_err = err > *max_err ? err : *max_err;
 
-    for(rocblas_int b = 0; b < bc; ++b)
-    {
         // error is ||hS - hSres||
         err = norm_error('F', 1, hNsv[b][0], 1, hS[b] + offset[b], hSres[b]); //WORKAROUND
         *max_err = err > *max_err ? err : *max_err;
@@ -464,6 +483,27 @@ void gesvdx_getError(const rocblas_handle handle,
         // Check the singular vectors if required
         if(hinfo[b][0] == 0 && (left_svect != rocblas_svect_none || right_svect != rocblas_svect_none))
         {
+            // U and V should be orthonormal, if they are then U^T*U and V*V^T should be the identity
+            if(nn > 0)
+            {
+                std::vector<T> UUres(nn * nn, 0.0);
+                std::vector<T> VVres(nn * nn, 0.0);
+                std::vector<T> I(nn * nn, 0.0);
+
+                for(rocblas_int i = 0; i < nn; i++)
+                    I[i + i * nn] = T(1);
+
+                cpu_gemm(rocblas_operation_conjugate_transpose, rocblas_operation_none, nn, nn, m,
+                         T(1), hUres[b], ldures, hUres[b], ldures, T(0), UUres.data(), nn);
+                err = norm_error('F', nn, nn, nn, I.data(), UUres.data());
+                *max_errv = err > *max_errv ? err : *max_errv;
+
+                cpu_gemm(rocblas_operation_none, rocblas_operation_conjugate_transpose, nn, nn, n,
+                         T(1), hVres[b], ldvres, hVres[b], ldvres, T(0), VVres.data(), nn);
+                err = norm_error('F', nn, nn, nn, I.data(), VVres.data());
+                *max_errv = err > *max_errv ? err : *max_errv;
+            }
+
             err = 0;
             // check singular vectors implicitly (A*v_k = s_k*u_k)
             for(rocblas_int k = 0; k < hNsv[b][0]; ++k)
@@ -627,25 +667,28 @@ void testing_gesvdx(Arguments& argus)
     char leftvC = argus.get<char>("left_svect");
     char rightvC = argus.get<char>("right_svect");
     char srangeC = argus.get<char>("srange");
-    rocblas_int m = argus.get<rocblas_int>("m");
-    rocblas_int n = argus.get<rocblas_int>("n", m);
-    rocblas_int nn = std::min(m, n);
-    rocblas_int lda = argus.get<rocblas_int>("lda", m);
-    rocblas_int ldu = argus.get<rocblas_int>("ldu", m);
-    rocblas_int ldv = argus.get<rocblas_int>("ldv", nn);
-    rocblas_stride stA = argus.get<rocblas_stride>("strideA", lda * n);
-    rocblas_stride stS = argus.get<rocblas_stride>("strideS", nn);
-    rocblas_stride stF = argus.get<rocblas_stride>("strideF", nn);
-    rocblas_stride stU = argus.get<rocblas_stride>("strideU", ldu * nn);
-    rocblas_stride stV = argus.get<rocblas_stride>("strideV", ldv * n);
+    rocblas_svect leftv = char2rocblas_svect(leftvC);
+    rocblas_svect rightv = char2rocblas_svect(rightvC);
+    rocblas_srange srange = char2rocblas_srange(srangeC);
+
     S vl = S(argus.get<double>("vl", 0));
     S vu = S(argus.get<double>("vu", srangeC == 'V' ? 1 : 0));
     rocblas_int il = argus.get<rocblas_int>("il", srangeC == 'I' ? 1 : 0);
     rocblas_int iu = argus.get<rocblas_int>("iu", srangeC == 'I' ? 1 : 0);
 
-    rocblas_svect leftv = char2rocblas_svect(leftvC);
-    rocblas_svect rightv = char2rocblas_svect(rightvC);
-    rocblas_srange srange = char2rocblas_srange(srangeC);
+    rocblas_int m = argus.get<rocblas_int>("m");
+    rocblas_int n = argus.get<rocblas_int>("n", m);
+    rocblas_int nn = std::min(m, n);
+    rocblas_int nsv_max = (srange == rocblas_srange_index ? iu - il + 1 : nn);
+    rocblas_int lda = argus.get<rocblas_int>("lda", m);
+    rocblas_int ldu = argus.get<rocblas_int>("ldu", m);
+    rocblas_int ldv = argus.get<rocblas_int>("ldv", nsv_max);
+    rocblas_stride stA = argus.get<rocblas_stride>("strideA", lda * n);
+    rocblas_stride stS = argus.get<rocblas_stride>("strideS", nsv_max);
+    rocblas_stride stF = argus.get<rocblas_stride>("strideF", nn);
+    rocblas_stride stU = argus.get<rocblas_stride>("strideU", ldu * nsv_max);
+    rocblas_stride stV = argus.get<rocblas_stride>("strideV", ldv * n);
+
     rocblas_int bc = argus.batch_count;
     rocblas_int hot_calls = argus.iters;
 
@@ -699,7 +742,7 @@ void testing_gesvdx(Arguments& argus)
         if(rightv == rocblas_svect_none)
         {
             rightvT = rocblas_svect_singular;
-            ldvT = nn;
+            ldvT = nsv_max;
             mT = m;
             nT = n;
         }
@@ -715,16 +758,17 @@ void testing_gesvdx(Arguments& argus)
     size_t size_UT = 0;
     size_t size_VT = 0;
     size_t size_A = size_t(lda) * n;
-    size_t size_S = size_t(nn);
+    size_t size_S = size_t(nsv_max);
+    size_t size_S_cpu = size_t(nn);
     size_t size_V = size_t(ldv) * n;
-    size_t size_U = size_t(ldu) * nn;
+    size_t size_U = size_t(ldu) * nsv_max;
     size_t size_ifail = nn;
     if(argus.unit_check || argus.norm_check)
     {
         size_hifailRes = nn;
         size_VT = size_t(ldvT) * n;
-        size_UT = size_t(lduT) * nn;
-        size_hSres = nn;
+        size_UT = size_t(lduT) * nsv_max;
+        size_hSres = nsv_max;
         if(svects)
         {
             if(leftv == rocblas_svect_none)
@@ -750,6 +794,7 @@ void testing_gesvdx(Arguments& argus)
             }
         }
     }
+    rocblas_stride stS_cpu = size_S_cpu;
     rocblas_stride stUT = size_UT;
     rocblas_stride stVT = size_VT;
     rocblas_stride stUres = size_hUres;
@@ -760,7 +805,7 @@ void testing_gesvdx(Arguments& argus)
     // check invalid sizes
     bool invalid_size = (n < 0 || m < 0 || lda < m || ldu < 1 || ldv < 1 || bc < 0)
         || (leftv == rocblas_svect_singular && ldu < m)
-        || (rightv == rocblas_svect_singular && ldv < nn)
+        || (rightv == rocblas_svect_singular && ldv < nsv_max)
         || (srange == rocblas_srange_value && (vl < 0 || vl >= vu))
         || (srange == rocblas_srange_index && (il < 1 || iu < 0))
         || (srange == rocblas_srange_index && (iu > nn || (nn > 0 && il > iu)));
@@ -828,7 +873,7 @@ void testing_gesvdx(Arguments& argus)
 
     // memory allocations (all cases)
     // host
-    host_strided_batch_vector<S> hS(size_S, 1, stS, bc);
+    host_strided_batch_vector<S> hS(size_S_cpu, 1, stS_cpu, bc); // extra space for cpu_gesvd call
     host_strided_batch_vector<T> hV(size_V, 1, stV, bc);
     host_strided_batch_vector<T> hU(size_U, 1, stU, bc);
     host_strided_batch_vector<rocblas_int> hNsv(1, 1, 1, bc);
@@ -956,7 +1001,7 @@ void testing_gesvdx(Arguments& argus)
     {
         ROCSOLVER_TEST_CHECK(T, max_error, 2 * min(m, n));
         if(svects)
-            ROCSOLVER_TEST_CHECK(T, max_errorv, 2 * min(m, n));
+            ROCSOLVER_TEST_CHECK(T, max_errorv, 4 * min(m, n));
     }
 
     // output results for rocsolver-bench
