@@ -1,6 +1,29 @@
-/* ************************************************************************
- * Copyright (c) 2020-2023 Advanced Micro Devices, Inc.
- * ************************************************************************ */
+/* **************************************************************************
+ * Copyright (C) 2020-2023 Advanced Micro Devices, Inc. All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ *
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+ * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+ * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
+ * *************************************************************************/
 
 #pragma once
 
@@ -230,9 +253,11 @@ void stein_getError(const rocblas_handle handle,
 
     if(hInfo[0][0] == 0)
     {
+        rocblas_int nn = hNev[0][0];
+
         // check ifail
         err = 0;
-        for(int j = 0; j < hNev[0][0]; j++)
+        for(int j = 0; j < nn; j++)
         {
             EXPECT_EQ(hIfailRes[0][j], 0) << "j = " << j;
             if(hIfailRes[0][j] != 0)
@@ -242,10 +267,25 @@ void stein_getError(const rocblas_handle handle,
 
         // need to implicitly test eigenvectors due to non-uniqueness of eigenvectors under scaling
 
+        // Z should be orthonormal, if it is then Z^T*Z should be the identity
+        if(nn > 0)
+        {
+            std::vector<T> ZZres(nn * nn, 0.0);
+            std::vector<T> I(nn * nn, 0.0);
+
+            for(rocblas_int i = 0; i < nn; i++)
+                I[i + i * nn] = T(1);
+
+            cpu_gemm(rocblas_operation_conjugate_transpose, rocblas_operation_none, nn, nn, n, T(1),
+                     hZRes[0], ldz, hZRes[0], ldz, T(0), ZZres.data(), nn);
+            err = norm_error('F', nn, nn, nn, I.data(), ZZres.data());
+            *max_err = err > *max_err ? err : *max_err;
+        }
+
         // for each of the nev eigenvalues w_j, verify that the associated eigenvector is in the
         // null space of (A - w_i * I)
         T alpha, t1, t2;
-        for(int j = 0; j < hNev[0][0]; j++)
+        for(int j = 0; j < nn; j++)
         {
             for(int i = 0; i < n; i++)
             {
@@ -265,7 +305,7 @@ void stein_getError(const rocblas_handle handle,
 
         // error is then ||hZ - hZRes|| / ||hZ||
         // using frobenius norm
-        err = norm_error('F', n, hNev[0][0], ldz, hZ[0], hZRes[0]);
+        err = norm_error('F', n, nn, ldz, hZ[0], hZRes[0]);
         *max_err = err > *max_err ? err : *max_err;
     }
     else
