@@ -1,6 +1,29 @@
-/* ************************************************************************
- * Copyright (c) 2021-2023 Advanced Micro Devices, Inc.
- * ************************************************************************ */
+/* **************************************************************************
+ * Copyright (C) 2021-2023 Advanced Micro Devices, Inc. All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ *
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+ * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+ * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
+ * *************************************************************************/
 
 #pragma once
 
@@ -386,43 +409,63 @@ inline void read_matrix(const std::string filenameS,
                         const rocblas_int lda)
 {
     const char* filename = filenameS.c_str();
-    FILE* mat;
-    mat = fopen(filename, "r");
-    rocblas_int v;
+    int const idebug = 0;
+    if(idebug >= 1)
+    {
+        printf("filename=%s, m=%d, n=%d, lda=%d\n", filename, m, n, lda);
+    }
+
+    FILE* mat = fopen(filename, "r");
 
     if(mat == NULL)
         throw std::invalid_argument(
             fmt::format("Error: Could not open file {} with test data...", filename));
 
+    rewind(mat);
+
     for(rocblas_int i = 0; i < m; ++i)
     {
         for(rocblas_int j = 0; j < n; ++j)
         {
+            rocblas_int v;
             int read = fscanf(mat, "%d", &v);
-            assert(read == 1);
+            if(read != 1)
+                throw std::out_of_range(
+                    fmt::format("Error: Could not read element {},{} from file {}", i, j, filename));
             A[i + j * lda] = v;
         }
     }
 
-    fclose(mat);
+    if(fclose(mat) != 0)
+    {
+        throw std::invalid_argument(
+            fmt::format("Error: Could not close file {} with test data...", filename));
+    }
 }
 inline void read_last(const std::string filenameS, rocblas_int* A)
 {
     const char* filename = filenameS.c_str();
-    FILE* mat;
-    mat = fopen(filename, "r");
-    rocblas_int v;
+    FILE* mat = fopen(filename, "r");
 
     if(mat == NULL)
         throw std::invalid_argument(
             fmt::format("Error: Could not open file {} with test data...", filename));
 
+    rewind(mat);
+
+    rocblas_int v;
     while(fscanf(mat, "%d", &v) == 1)
     {
         // do nothing
     }
 
     *A = v;
+
+    if(fclose(mat) != 0)
+    {
+        throw std::invalid_argument(
+            fmt::format("Error: Could not close file {} with test data...", filename));
+    }
 }
 
 // singles:
@@ -433,25 +476,38 @@ inline void read_matrix(const std::string filenameS,
                         const rocblas_int lda)
 {
     const char* filename = filenameS.c_str();
-    FILE* mat;
-    mat = fopen(filename, "r");
-    float v;
+    int const idebug = 0;
+    if(idebug >= 1)
+    {
+        printf("filename=%s, m=%d, n=%d, lda=%d\n", filename, m, n, lda);
+    }
+
+    FILE* mat = fopen(filename, "r");
 
     if(mat == NULL)
         throw std::invalid_argument(
             fmt::format("Error: Could not open file {} with test data...", filename));
 
+    rewind(mat);
+
     for(rocblas_int i = 0; i < m; ++i)
     {
         for(rocblas_int j = 0; j < n; ++j)
         {
+            float v;
             int read = fscanf(mat, "%g", &v);
-            assert(read == 1);
+            if(read != 1)
+                throw std::out_of_range(
+                    fmt::format("Error: Could not read element {},{} from file {}", i, j, filename));
             A[i + j * lda] = v;
         }
     }
 
-    fclose(mat);
+    if(fclose(mat) != 0)
+    {
+        throw std::invalid_argument(
+            fmt::format("Error: Could not close file {} with test data...", filename));
+    }
 }
 
 // doubles:
@@ -462,23 +518,35 @@ inline void read_matrix(const std::string filenameS,
                         const rocblas_int lda)
 {
     const char* filename = filenameS.c_str();
-    FILE* mat;
-    mat = fopen(filename, "r");
-    double v;
+    int const idebug = 0;
+    if(idebug >= 1)
+    {
+        printf("filename=%s, m=%d, n=%d, lda=%d\n", filename, m, n, lda);
+    }
+    FILE* mat = fopen(filename, "r");
 
     if(mat == NULL)
         throw std::invalid_argument(
             fmt::format("Error: Could not open file {} with test data...", filename));
 
+    rewind(mat);
+
     for(rocblas_int i = 0; i < m; ++i)
     {
         for(rocblas_int j = 0; j < n; ++j)
         {
+            double v;
             int read = fscanf(mat, "%lg", &v);
-            assert(read == 1);
+            if(read != 1)
+                throw std::out_of_range(
+                    fmt::format("Error: Could not read element {},{} from file {}", i, j, filename));
             A[i + j * lda] = v;
         }
     }
 
-    fclose(mat);
+    if(fclose(mat) != 0)
+    {
+        throw std::invalid_argument(
+            fmt::format("Error: Could not close file {} with test data...", filename));
+    }
 }
