@@ -1,6 +1,29 @@
-/* ************************************************************************
- * Copyright (c) 2019-2023 Advanced Micro Devices, Inc.
- * ************************************************************************ */
+/* **************************************************************************
+ * Copyright (C) 2019-2023 Advanced Micro Devices, Inc. All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ *
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+ * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+ * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
+ * *************************************************************************/
 
 #pragma once
 
@@ -764,28 +787,28 @@ void rocsolver_trsm_mem(const rocblas_side side,
     This is blocked implementation that calls the internal forward/backward subtitution kernels
     to solve the diagonal blocks, and uses gemm to update the right/left -hand-sides **/
 template <bool BATCHED, bool STRIDED, typename T, typename U>
-void rocsolver_trsm_lower(rocblas_handle handle,
-                          const rocblas_side side,
-                          const rocblas_operation trans,
-                          const rocblas_diagonal diag,
-                          const rocblas_int m,
-                          const rocblas_int n,
-                          U A,
-                          const rocblas_int shiftA,
-                          const rocblas_int inca,
-                          const rocblas_int lda,
-                          const rocblas_stride strideA,
-                          U B,
-                          const rocblas_int shiftB,
-                          const rocblas_int incb,
-                          const rocblas_int ldb,
-                          const rocblas_stride strideB,
-                          const rocblas_int batch_count,
-                          const bool optim_mem,
-                          void* work1,
-                          void* work2,
-                          void* work3,
-                          void* work4)
+rocblas_status rocsolver_trsm_lower(rocblas_handle handle,
+                                    const rocblas_side side,
+                                    const rocblas_operation trans,
+                                    const rocblas_diagonal diag,
+                                    const rocblas_int m,
+                                    const rocblas_int n,
+                                    U A,
+                                    const rocblas_int shiftA,
+                                    const rocblas_int inca,
+                                    const rocblas_int lda,
+                                    const rocblas_stride strideA,
+                                    U B,
+                                    const rocblas_int shiftB,
+                                    const rocblas_int incb,
+                                    const rocblas_int ldb,
+                                    const rocblas_stride strideB,
+                                    const rocblas_int batch_count,
+                                    const bool optim_mem,
+                                    void* work1,
+                                    void* work2,
+                                    void* work3,
+                                    void* work4)
 {
     ROCSOLVER_ENTER("trsm_lower", "side:", side, "trans:", trans, "diag:", diag, "m:", m, "n:", n,
                     "shiftA:", shiftA, "lda:", lda, "shiftB:", shiftB, "ldb:", ldb,
@@ -819,12 +842,21 @@ void rocsolver_trsm_lower(rocblas_handle handle,
 
     if(blk == 0)
     {
-        rocblasCall_trsm(handle, side, rocblas_fill_lower, trans, diag, m, n, &one, A, shiftA, lda,
-                         strideA, B, shiftB, ldb, strideB, batch_count, optim_mem, work1, work2,
-                         work3, work4);
-        return;
+        return rocblasCall_trsm(handle, side, rocblas_fill_lower, trans, diag, m, n, &one, A,
+                                shiftA, lda, strideA, B, shiftB, ldb, strideB, batch_count,
+                                optim_mem, work1, work2, work3, work4);
     }
 
+    // TODO: Some architectures require synchronization between rocSOLVER and rocBLAS kernels; more investigation needed
+    int device;
+    HIP_CHECK(hipGetDevice(&device));
+    hipDeviceProp_t deviceProperties;
+    HIP_CHECK(hipGetDeviceProperties(&deviceProperties, device));
+    std::string deviceFullString(deviceProperties.gcnArchName);
+    std::string deviceString = deviceFullString.substr(0, deviceFullString.find(":"));
+    bool do_sync = (deviceString.find("gfx940") != std::string::npos
+                    || deviceString.find("gfx941") != std::string::npos);
+
     // ****** MAIN LOOP ***********
     if(isleft)
     {
@@ -854,13 +886,16 @@ void rocsolver_trsm_lower(rocblas_handle handle,
                 offB = idx2D(j, 0, incb, ldb);
                 FORWARD_SUBSTITUTIONS;
 
+                if(do_sync)
+                    HIP_CHECK(hipStreamSynchronize(stream));
+
                 // update right hand sides
-                rocsolver_gemm<BATCHED, STRIDED, T>(
+                ROCBLAS_CHECK(rocsolver_gemm<BATCHED, STRIDED, T>(
                     handle, rocblas_operation_none, rocblas_operation_none, m - nextpiv, n, blk,
                     &minone, A, shiftA + idx2D(nextpiv, j, inca, lda), inca, lda, strideA, B,
                     shiftB + idx2D(j, 0, incb, ldb), incb, ldb, strideB, &one, B,
                     shiftB + idx2D(nextpiv, 0, incb, ldb), incb, ldb, strideB, batch_count,
-                    (T**)nullptr);
+                    (T**)nullptr));
 
                 j = nextpiv;
             }
@@ -895,12 +930,15 @@ void rocsolver_trsm_lower(rocblas_handle handle,
                 offB = idx2D(m - nextpiv, 0, incb, ldb);
                 BACKWARD_SUBSTITUTIONS;
 
+                if(do_sync)
+                    HIP_CHECK(hipStreamSynchronize(stream));
+
                 // update right hand sides
-                rocsolver_gemm<BATCHED, STRIDED, T>(
+                ROCBLAS_CHECK(rocsolver_gemm<BATCHED, STRIDED, T>(
                     handle, trans, rocblas_operation_none, m - nextpiv, n, blk, &minone, A,
                     shiftA + idx2D(m - nextpiv, 0, inca, lda), inca, lda, strideA, B,
                     shiftB + idx2D(m - nextpiv, 0, incb, ldb), incb, ldb, strideB, &one, B,
-                    shiftB + idx2D(0, 0, incb, ldb), incb, ldb, strideB, batch_count, (T**)nullptr);
+                    shiftB + idx2D(0, 0, incb, ldb), incb, ldb, strideB, batch_count, (T**)nullptr));
 
                 j = nextpiv;
             }
@@ -948,12 +986,15 @@ void rocsolver_trsm_lower(rocblas_handle handle,
                 offB = idx2D(0, n - nextpiv, incb, ldb);
                 BACKWARD_SUBSTITUTIONS;
 
+                if(do_sync)
+                    HIP_CHECK(hipStreamSynchronize(stream));
+
                 // update left hand sides
-                rocsolver_gemm<BATCHED, STRIDED, T>(
+                ROCBLAS_CHECK(rocsolver_gemm<BATCHED, STRIDED, T>(
                     handle, rocblas_operation_none, rocblas_operation_none, m, n - nextpiv, blk,
                     &minone, B, shiftB + idx2D(0, n - nextpiv, incb, ldb), incb, ldb, strideB, A,
                     shiftA + idx2D(n - nextpiv, 0, inca, lda), inca, lda, strideA, &one, B,
-                    shiftB + idx2D(0, 0, incb, ldb), incb, ldb, strideB, batch_count, (T**)nullptr);
+                    shiftB + idx2D(0, 0, incb, ldb), incb, ldb, strideB, batch_count, (T**)nullptr));
 
                 j = nextpiv;
             }
@@ -988,13 +1029,16 @@ void rocsolver_trsm_lower(rocblas_handle handle,
                 offB = idx2D(0, j, incb, ldb);
                 FORWARD_SUBSTITUTIONS;
 
+                if(do_sync)
+                    HIP_CHECK(hipStreamSynchronize(stream));
+
                 // update left hand sides
-                rocsolver_gemm<BATCHED, STRIDED, T>(
+                ROCBLAS_CHECK(rocsolver_gemm<BATCHED, STRIDED, T>(
                     handle, rocblas_operation_none, trans, m, n - nextpiv, blk, &minone, B,
                     shiftB + idx2D(0, j, incb, ldb), incb, ldb, strideB, A,
                     shiftA + idx2D(nextpiv, j, inca, lda), inca, lda, strideA, &one, B,
                     shiftB + idx2D(0, nextpiv, incb, ldb), incb, ldb, strideB, batch_count,
-                    (T**)nullptr);
+                    (T**)nullptr));
 
                 j = nextpiv;
             }
@@ -1013,6 +1057,8 @@ void rocsolver_trsm_lower(rocblas_handle handle,
             FORWARD_SUBSTITUTIONS;
         }
     }
+
+    return rocblas_status_success;
 }
 
 /** Internal TRSM (upper case):
@@ -1024,28 +1070,28 @@ void rocsolver_trsm_lower(rocblas_handle handle,
     This is blocked implementation that calls the internal forward/backward subtitution kernels
     to solve the diagonal blocks, and uses gemm to update the right/left -hand-sides **/
 template <bool BATCHED, bool STRIDED, typename T, typename U>
-void rocsolver_trsm_upper(rocblas_handle handle,
-                          const rocblas_side side,
-                          const rocblas_operation trans,
-                          const rocblas_diagonal diag,
-                          const rocblas_int m,
-                          const rocblas_int n,
-                          U A,
-                          const rocblas_int shiftA,
-                          const rocblas_int inca,
-                          const rocblas_int lda,
-                          const rocblas_stride strideA,
-                          U B,
-                          const rocblas_int shiftB,
-                          const rocblas_int incb,
-                          const rocblas_int ldb,
-                          const rocblas_stride strideB,
-                          const rocblas_int batch_count,
-                          const bool optim_mem,
-                          void* work1,
-                          void* work2,
-                          void* work3,
-                          void* work4)
+rocblas_status rocsolver_trsm_upper(rocblas_handle handle,
+                                    const rocblas_side side,
+                                    const rocblas_operation trans,
+                                    const rocblas_diagonal diag,
+                                    const rocblas_int m,
+                                    const rocblas_int n,
+                                    U A,
+                                    const rocblas_int shiftA,
+                                    const rocblas_int inca,
+                                    const rocblas_int lda,
+                                    const rocblas_stride strideA,
+                                    U B,
+                                    const rocblas_int shiftB,
+                                    const rocblas_int incb,
+                                    const rocblas_int ldb,
+                                    const rocblas_stride strideB,
+                                    const rocblas_int batch_count,
+                                    const bool optim_mem,
+                                    void* work1,
+                                    void* work2,
+                                    void* work3,
+                                    void* work4)
 {
     ROCSOLVER_ENTER("trsm_upper", "side:", side, "trans:", trans, "diag:", diag, "m:", m, "n:", n,
                     "shiftA:", shiftA, "lda:", lda, "shiftB:", shiftB, "ldb:", ldb,
@@ -1079,12 +1125,21 @@ void rocsolver_trsm_upper(rocblas_handle handle,
 
     if(blk == 0)
     {
-        rocblasCall_trsm(handle, side, rocblas_fill_upper, trans, diag, m, n, &one, A, shiftA, lda,
-                         strideA, B, shiftB, ldb, strideB, batch_count, optim_mem, work1, work2,
-                         work3, work4);
-        return;
+        return rocblasCall_trsm(handle, side, rocblas_fill_upper, trans, diag, m, n, &one, A,
+                                shiftA, lda, strideA, B, shiftB, ldb, strideB, batch_count,
+                                optim_mem, work1, work2, work3, work4);
     }
 
+    // TODO: Some architectures require synchronization between rocSOLVER and rocBLAS kernels; more investigation needed
+    int device;
+    HIP_CHECK(hipGetDevice(&device));
+    hipDeviceProp_t deviceProperties;
+    HIP_CHECK(hipGetDeviceProperties(&deviceProperties, device));
+    std::string deviceFullString(deviceProperties.gcnArchName);
+    std::string deviceString = deviceFullString.substr(0, deviceFullString.find(":"));
+    bool do_sync = (deviceString.find("gfx940") != std::string::npos
+                    || deviceString.find("gfx941") != std::string::npos);
+
     // ****** MAIN LOOP ***********
     if(isleft)
     {
@@ -1114,13 +1169,16 @@ void rocsolver_trsm_upper(rocblas_handle handle,
                 offB = idx2D(j, 0, incb, ldb);
                 FORWARD_SUBSTITUTIONS;
 
+                if(do_sync)
+                    HIP_CHECK(hipStreamSynchronize(stream));
+
                 // update right hand sides
-                rocsolver_gemm<BATCHED, STRIDED, T>(
+                ROCBLAS_CHECK(rocsolver_gemm<BATCHED, STRIDED, T>(
                     handle, trans, rocblas_operation_none, m - nextpiv, n, blk, &minone, A,
                     shiftA + idx2D(j, nextpiv, inca, lda), inca, lda, strideA, B,
                     shiftB + idx2D(j, 0, incb, ldb), incb, ldb, strideB, &one, B,
                     shiftB + idx2D(nextpiv, 0, incb, ldb), incb, ldb, strideB, batch_count,
-                    (T**)nullptr);
+                    (T**)nullptr));
 
                 j = nextpiv;
             }
@@ -1155,12 +1213,15 @@ void rocsolver_trsm_upper(rocblas_handle handle,
                 offB = idx2D(m - nextpiv, 0, incb, ldb);
                 BACKWARD_SUBSTITUTIONS;
 
+                if(do_sync)
+                    HIP_CHECK(hipStreamSynchronize(stream));
+
                 // update right hand sides
-                rocsolver_gemm<BATCHED, STRIDED, T>(
+                ROCBLAS_CHECK(rocsolver_gemm<BATCHED, STRIDED, T>(
                     handle, rocblas_operation_none, rocblas_operation_none, m - nextpiv, n, blk,
                     &minone, A, shiftA + idx2D(0, m - nextpiv, inca, lda), inca, lda, strideA, B,
                     shiftB + idx2D(m - nextpiv, 0, incb, ldb), incb, ldb, strideB, &one, B,
-                    shiftB + idx2D(0, 0, incb, ldb), incb, ldb, strideB, batch_count, (T**)nullptr);
+                    shiftB + idx2D(0, 0, incb, ldb), incb, ldb, strideB, batch_count, (T**)nullptr));
 
                 j = nextpiv;
             }
@@ -1208,12 +1269,15 @@ void rocsolver_trsm_upper(rocblas_handle handle,
                 offB = idx2D(0, n - nextpiv, incb, ldb);
                 BACKWARD_SUBSTITUTIONS;
 
+                if(do_sync)
+                    HIP_CHECK(hipStreamSynchronize(stream));
+
                 // update left hand sides
-                rocsolver_gemm<BATCHED, STRIDED, T>(
+                ROCBLAS_CHECK(rocsolver_gemm<BATCHED, STRIDED, T>(
                     handle, rocblas_operation_none, trans, m, n - nextpiv, blk, &minone, B,
                     shiftB + idx2D(0, n - nextpiv, incb, ldb), incb, ldb, strideB, A,
                     shiftA + idx2D(0, n - nextpiv, inca, lda), inca, lda, strideA, &one, B,
-                    shiftB + idx2D(0, 0, incb, ldb), incb, ldb, strideB, batch_count, (T**)nullptr);
+                    shiftB + idx2D(0, 0, incb, ldb), incb, ldb, strideB, batch_count, (T**)nullptr));
 
                 j = nextpiv;
             }
@@ -1248,13 +1312,16 @@ void rocsolver_trsm_upper(rocblas_handle handle,
                 offB = idx2D(0, j, incb, ldb);
                 FORWARD_SUBSTITUTIONS;
 
+                if(do_sync)
+                    HIP_CHECK(hipStreamSynchronize(stream));
+
                 // update left hand sides
-                rocsolver_gemm<BATCHED, STRIDED, T>(
+                ROCBLAS_CHECK(rocsolver_gemm<BATCHED, STRIDED, T>(
                     handle, rocblas_operation_none, rocblas_operation_none, m, n - nextpiv, blk,
                     &minone, B, shiftB + idx2D(0, j, incb, ldb), incb, ldb, strideB, A,
                     shiftA + idx2D(j, nextpiv, inca, lda), inca, lda, strideA, &one, B,
                     shiftB + idx2D(0, nextpiv, incb, ldb), incb, ldb, strideB, batch_count,
-                    (T**)nullptr);
+                    (T**)nullptr));
 
                 j = nextpiv;
             }
@@ -1273,6 +1340,8 @@ void rocsolver_trsm_upper(rocblas_handle handle,
             FORWARD_SUBSTITUTIONS;
         }
     }
+
+    return rocblas_status_success;
 }
 
 /*************************************************************
@@ -1280,57 +1349,57 @@ void rocsolver_trsm_upper(rocblas_handle handle,
 *************************************************************/
 
 template <bool BATCHED, bool STRIDED, typename T, typename U>
-inline void rocsolver_trsm_lower(rocblas_handle handle,
-                                 const rocblas_side side,
-                                 const rocblas_operation trans,
-                                 const rocblas_diagonal diag,
-                                 const rocblas_int m,
-                                 const rocblas_int n,
-                                 U A,
-                                 const rocblas_int shiftA,
-                                 const rocblas_int lda,
-                                 const rocblas_stride strideA,
-                                 U B,
-                                 const rocblas_int shiftB,
-                                 const rocblas_int ldb,
-                                 const rocblas_stride strideB,
-                                 const rocblas_int batch_count,
-                                 const bool optim_mem,
-                                 void* work1,
-                                 void* work2,
-                                 void* work3,
-                                 void* work4)
+inline rocblas_status rocsolver_trsm_lower(rocblas_handle handle,
+                                           const rocblas_side side,
+                                           const rocblas_operation trans,
+                                           const rocblas_diagonal diag,
+                                           const rocblas_int m,
+                                           const rocblas_int n,
+                                           U A,
+                                           const rocblas_int shiftA,
+                                           const rocblas_int lda,
+                                           const rocblas_stride strideA,
+                                           U B,
+                                           const rocblas_int shiftB,
+                                           const rocblas_int ldb,
+                                           const rocblas_stride strideB,
+                                           const rocblas_int batch_count,
+                                           const bool optim_mem,
+                                           void* work1,
+                                           void* work2,
+                                           void* work3,
+                                           void* work4)
 {
-    rocsolver_trsm_lower<BATCHED, STRIDED, T>(handle, side, trans, diag, m, n, A, shiftA, 1, lda,
-                                              strideA, B, shiftB, 1, ldb, strideB, batch_count,
-                                              optim_mem, work1, work2, work3, work4);
+    return rocsolver_trsm_lower<BATCHED, STRIDED, T>(
+        handle, side, trans, diag, m, n, A, shiftA, 1, lda, strideA, B, shiftB, 1, ldb, strideB,
+        batch_count, optim_mem, work1, work2, work3, work4);
 }
 
 template <bool BATCHED, bool STRIDED, typename T, typename U>
-inline void rocsolver_trsm_upper(rocblas_handle handle,
-                                 const rocblas_side side,
-                                 const rocblas_operation trans,
-                                 const rocblas_diagonal diag,
-                                 const rocblas_int m,
-                                 const rocblas_int n,
-                                 U A,
-                                 const rocblas_int shiftA,
-                                 const rocblas_int lda,
-                                 const rocblas_stride strideA,
-                                 U B,
-                                 const rocblas_int shiftB,
-                                 const rocblas_int ldb,
-                                 const rocblas_stride strideB,
-                                 const rocblas_int batch_count,
-                                 const bool optim_mem,
-                                 void* work1,
-                                 void* work2,
-                                 void* work3,
-                                 void* work4)
+inline rocblas_status rocsolver_trsm_upper(rocblas_handle handle,
+                                           const rocblas_side side,
+                                           const rocblas_operation trans,
+                                           const rocblas_diagonal diag,
+                                           const rocblas_int m,
+                                           const rocblas_int n,
+                                           U A,
+                                           const rocblas_int shiftA,
+                                           const rocblas_int lda,
+                                           const rocblas_stride strideA,
+                                           U B,
+                                           const rocblas_int shiftB,
+                                           const rocblas_int ldb,
+                                           const rocblas_stride strideB,
+                                           const rocblas_int batch_count,
+                                           const bool optim_mem,
+                                           void* work1,
+                                           void* work2,
+                                           void* work3,
+                                           void* work4)
 {
-    rocsolver_trsm_upper<BATCHED, STRIDED, T>(handle, side, trans, diag, m, n, A, shiftA, 1, lda,
-                                              strideA, B, shiftB, 1, ldb, strideB, batch_count,
-                                              optim_mem, work1, work2, work3, work4);
+    return rocsolver_trsm_upper<BATCHED, STRIDED, T>(
+        handle, side, trans, diag, m, n, A, shiftA, 1, lda, strideA, B, shiftB, 1, ldb, strideB,
+        batch_count, optim_mem, work1, work2, work3, work4);
 }
 
 /*************************************************************
@@ -1344,7 +1413,7 @@ inline void rocsolver_trsm_upper(rocblas_handle handle,
         size_t* size_work2, size_t* size_work3, size_t* size_work4, bool* optim_mem, \
         bool inblocked, const rocblas_int, const rocblas_int)
 #define INSTANTIATE_TRSM_LOWER(BATCHED, STRIDED, T, U)                                      \
-    template void rocsolver_trsm_lower<BATCHED, STRIDED, T, U>(                             \
+    template rocblas_status rocsolver_trsm_lower<BATCHED, STRIDED, T, U>(                   \
         rocblas_handle handle, const rocblas_side side, const rocblas_operation trans,      \
         const rocblas_diagonal diag, const rocblas_int m, const rocblas_int n, U A,         \
         const rocblas_int shiftA, const rocblas_int lda, const rocblas_stride strideA, U B, \
@@ -1352,7 +1421,7 @@ inline void rocsolver_trsm_upper(rocblas_handle handle,
         const rocblas_int batch_count, const bool optim_mem, void* work1, void* work2,      \
         void* work3, void* work4)
 #define INSTANTIATE_TRSM_UPPER(BATCHED, STRIDED, T, U)                                      \
-    template void rocsolver_trsm_upper<BATCHED, STRIDED, T, U>(                             \
+    template rocblas_status rocsolver_trsm_upper<BATCHED, STRIDED, T, U>(                   \
         rocblas_handle handle, const rocblas_side side, const rocblas_operation trans,      \
         const rocblas_diagonal diag, const rocblas_int m, const rocblas_int n, U A,         \
         const rocblas_int shiftA, const rocblas_int lda, const rocblas_stride strideA, U B, \
